@@ -86,6 +86,8 @@ struct pa_card {
 
     pa_suspend_cause_t suspend_cause;
 
+    bool linked;
+
     void *userdata;
 
     int (*set_profile)(pa_card *c, pa_card_profile *profile);
@@ -99,15 +101,11 @@ typedef struct pa_card_new_data {
     pa_module *module;
 
     pa_hashmap *profiles;
-    char *active_profile;
-
     pa_hashmap *ports;
     pa_device_port *preferred_input_port;
     pa_device_port *preferred_output_port;
 
     bool namereg_fail:1;
-
-    bool save_profile:1;
 } pa_card_new_data;
 
 typedef struct {
@@ -123,11 +121,17 @@ void pa_card_profile_set_available(pa_card_profile *c, pa_available_t available)
 
 pa_card_new_data *pa_card_new_data_init(pa_card_new_data *data);
 void pa_card_new_data_set_name(pa_card_new_data *data, const char *name);
-void pa_card_new_data_set_profile(pa_card_new_data *data, const char *profile);
 void pa_card_new_data_set_preferred_port(pa_card_new_data *data, pa_direction_t direction, pa_device_port *port);
 void pa_card_new_data_done(pa_card_new_data *data);
 
 pa_card *pa_card_new(pa_core *c, pa_card_new_data *data);
+
+/* Select the initial card profile according to the configured policies. This
+ * must be called between pa_card_new() and pa_card_put(), after the port and
+ * profile availablities have been initialized. */
+void pa_card_choose_initial_profile(pa_card *card);
+
+void pa_card_put(pa_card *c);
 void pa_card_free(pa_card *c);
 
 void pa_card_add_profile(pa_card *c, pa_card_profile *profile);
